@@ -28,6 +28,14 @@
 getMommy<-function(tree,
                    N,
                    curr=NULL){
+
+  if(!identical(tree$tip.label,tips(tree,(Ntip(tree)+1)))){
+    data.frame(tree$tip.label,N=seq(1,Ntip(tree)))->dftips
+    tree$tip.label<-tips(tree,(Ntip(tree)+1))
+    data.frame(dftips,Nor=match(dftips[,1],tree$tip.label))->dftips
+    tree$edge[match(dftips[,2],tree$edge[,2]),2]<-dftips[,3]
+  }
+
   N->node
   if(is.null(curr)) curr<-vector()
   daughters<-tree$edge[which(tree$edge[,2]==node),1]
